#!/usr/bin/perl
#

use strict;

my $CLOCK_RATE = 200000;

my $fileIn = shift;
$fileIn =~ /(.*)\.hex/;
my $baseName = $1;
my $fileOut = "$baseName.svf";

open FHI, "<$fileIn" or die "ERROR: Can't open $fileIn";
open FHO, ">$fileOut" or die "ERROR: Can't open $fileOut";

initJtag();
erase_bs();
write_bs();

close FHI;
close FHO;

`svf2xsvf.exe -d -i $baseName.svf -o $baseName.xsvf`;

###################################################################
# Flip the bit ordering of a 
###################################################################
sub flipBits {
  my $in = shift @_;
  my $data = hex($in);
  my $newData = 0;
  $newData = $newData +   1 if (0+$data & 512);
  $newData = $newData +   2 if (0+$data & 256);
  $newData = $newData +   4 if (0+$data & 128);
  $newData = $newData +   8 if (0+$data &  64);
  $newData = $newData +  16 if (0+$data &  32);
  $newData = $newData +  32 if (0+$data &  16);
  $newData = $newData +  64 if (0+$data &   8);
  $newData = $newData + 128 if (0+$data &   4);
  $newData = $newData + 256 if (0+$data &   2);
  $newData = $newData + 512 if (0+$data &   1);
  my $returnValue = sprintf "%03x", $newData;
  return $returnValue;
}

###################################################################
# Initialize JTAG chain and put in user mode
###################################################################
sub initJtag {
  print FHO "// Initializing Jtag chain and going to user mode\n";
  print FHO "TRST OFF;\n";
  print FHO "ENDIR IDLE;\n";
  print FHO "ENDDR IDLE;\n";
  print FHO "STATE RESET;\n";
  print FHO "STATE IDLE;\n";
  print FHO "FREQUENCY 1E6 HZ;\n";
  print FHO "TIR 0 ;\n";;
  print FHO "HIR 8 TDI (ff) SMASK (ff) ;\n";
  print FHO "HDR 1 TDI (00) SMASK (01) ;\n";
  print FHO "TDR 0 ;\n";
  print FHO "SIR 6 TDI (02) ;\n";
}

###################################################################
# Put JTAG data
###################################################################
sub put_jtag {
  my $data = shift;
  while ($data ne "") {
    $data =~ /^(..)(.*)/;
    my $string_piece = "2$1";
    $data = $2;
    print FHO "SDR 13 TDI (0$string_piece) SMASK (1fff);\n";
  }
}

###################################################################
# Write a register in the port space
###################################################################
sub write_reg {
   my $reg = shift @_;
   my $data = shift @_;
   put_jtag("57$reg$data");
   return;
}

###################################################################
# Burst a register in the port space
###################################################################
sub burst_reg {
   my $reg = shift @_;
   my $data = shift @_;
   my $size = sprintf "%04x", int(length($data)/2);
   put_jtag("42$reg$size$data");
   return;
}

###################################################################
# Erase sector
###################################################################
sub erase_sector {
  my $addr = shift @_;
  print FHO "// Erasing sector $addr\n";
  $addr =~ /^(..)(..)(..)/;
  my $addrh = $1;
  my $addrm = $2;
  my $addrl = $3;
  write_reg("30", "00");
  write_reg("31", "7c");
  write_reg("31", $addrh);
  write_reg("31", $addrm);
  write_reg("31", $addrl);
  write_reg("30", "ff");
  my $numClocks = int (5 * $CLOCK_RATE);
  print FHO "RUNTEST $numClocks TCK;\n";
}

###################################################################
# Erase bitstream image 1
###################################################################
sub erase_bs {
  erase_sector("100000");
  erase_sector("140000");
  erase_sector("180000");
  erase_sector("1c0000");
}

###################################################################
# Write a page to the flash
###################################################################
sub page_to_buffer {
  my $addr = shift @_;
  $addr =~ /^(..)(..)(..)/;
  my $addrh = $1;
  my $addrm = $2;
  my $addrl = $3;
  write_reg("30", "00");
  write_reg("31", "53");
  write_reg("31", $addrh);
  write_reg("31", $addrm);
  write_reg("31", $addrl);
  write_reg("30", "FF");
  my $numClocks = int (0.0002 * $CLOCK_RATE);
  print FHO "RUNTEST $numClocks TCK;\n";
}

###################################################################
# Write a page to the flash
###################################################################
sub write_buffer {
  my $addr = shift @_;
  my $data = shift @_;
  $addr =~ /^(..)(..)(..)/;
  my $addrh = $1;
  my $addrm = $2;
  my $addrl = $3;
  write_reg("30", "00");
  write_reg("31", "84");
  write_reg("31", $addrh);
  write_reg("31", $addrm);
  write_reg("31", $addrl);
  burst_reg("31", $data);
  write_reg("30", "FF");
}

###################################################################
# Program the buffer into the flash
###################################################################
sub buffer_to_page {
  my $addr = shift @_;
  $addr =~ /^(..)(..)(..)/;
  my $addrh = $1;
  my $addrm = $2;
  my $addrl = $3;
  write_reg("30", "00");
  write_reg("31", "88");
  write_reg("31", $addrh);
  write_reg("31", $addrm);
  write_reg("31", $addrl);
  write_reg("30", "FF");
  my $numClocks = int (0.006 * $CLOCK_RATE);
  print FHO "RUNTEST $numClocks TCK;\n";

}

###################################################################
# Write the bitstream image
###################################################################
sub write_bs {
  my $numBytes = -s $fileIn;
  my $numBytes_hex = sprintf "%02x", $numBytes;
  my $numPages = int ($numBytes / 1056) +1;
  my $kbSent = 0;
  my $checkValue = 0xffff;
  my $addr = 1048576; 
  for my $i (1 .. $numPages) {
    read FHI, my $myBytes, 1056;
    my $addr_hex = sprintf "%06x", $addr;
    print FHO "// Writing page for address 0x$addr_hex\n";
    page_to_buffer($addr_hex);
    write_buffer($addr_hex,$myBytes);
    buffer_to_page($addr_hex);
    $addr = $addr + 1024;
  }
}
