------------------------------------------------------------------------------
-- Copyright (c) 2008 Xilinx, Inc.
-- This design is confidential and proprietary of Xilinx, All Rights Reserved.
------------------------------------------------------------------------------
--   ____  ____
--  /   /\/   /
-- /___/  \  /   Vendor:        Xilinx
-- \   \   \/    Version:       1.0.0
--  \   \        Filename:      picoboot.vhd
--  /   /        Date Created:  April 22, 2008
-- /___/   /\    Last Modified: April 22, 2008
-- \   \  /  \
--  \___\/\___\
--
-- Devices:   Spartan-3 Generation FPGA
-- Purpose:   MultiBoot rebooter module
-- Contact:   crabill@xilinx.com
-- Reference: None
--
-- Revision History:
--   Rev 1.0.0 - (crabill) First created April 22, 2008
--
------------------------------------------------------------------------------
--
-- LIMITED WARRANTY AND DISCLAIMER. These designs are provided to you "as is".
-- Xilinx and its licensors make and you receive no warranties or conditions,
-- express, implied, statutory or otherwise, and Xilinx specifically disclaims
-- any implied warranties of merchantability, non-infringement, or fitness for
-- a particular purpose. Xilinx does not warrant that the functions contained
-- in these designs will meet your requirements, or that the operation of
-- these designs will be uninterrupted or error free, or that defects in the
-- designs will be corrected. Furthermore, Xilinx does not warrant or make any
-- representations regarding use or the results of the use of the designs in
-- terms of correctness, accuracy, reliability, or otherwise.
--
-- LIMITATION OF LIABILITY. In no event will Xilinx or its licensors be liable
-- for any loss of data, lost profits, cost or procurement of substitute goods
-- or services, or for any special, incidental, consequential, or indirect
-- damages arising from the use or operation of the designs or accompanying
-- documentation, however caused and on any theory of liability. This
-- limitation will apply even if Xilinx has been advised of the possibility
-- of such damage. This limitation shall apply not-withstanding the failure
-- of the essential purpose of any limited remedies herein.
--
------------------------------------------------------------------------------
-- Copyright (c) 2008 Xilinx, Inc.
-- This design is confidential and proprietary of Xilinx, All Rights Reserved.
------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.all;
use IEEE.STD_LOGIC_UNSIGNED.all;

library UNISIM;
use UNISIM.VCOMPONENTS.all;

entity PICOBOOT is
  port (
    INTERNAL_ADDR : in  std_logic_vector(31 downto 0);
    INTERNAL_MODE : in  std_logic_vector(2 downto 0);
    INTERNAL_VSEL : in  std_logic_vector(2 downto 0);
    INTERNAL_USE  : in  std_logic;
    REBOOT        : in  std_logic;
    CLK           : in  std_logic);
end PICOBOOT;

architecture RTL of PICOBOOT is

  constant DONE : std_logic_vector(4 downto 0) := "10101";
  signal state    : std_logic_vector(4 downto 0);
  signal data     : std_logic_vector(7 downto 0);
  signal write    : std_logic;
  signal ce       : std_logic;
  signal icap_ce  : std_logic;
  signal icap_data    : std_logic_vector(7 downto 0);
  signal new_mode_reg : std_logic_vector(7 downto 0);

begin

  ----------------------------------------------------------------------
  -- Instantiate the ICAP primitive.                                  --
  ----------------------------------------------------------------------

  icap_inst : ICAP_SPARTAN3A
    port map (
      I     => data,
      O     => open,
      WRITE => write,
      BUSY  => open,
      CLK   => CLK,
      CE    => ce );

  ----------------------------------------------------------------------
  -- Simple write-only FSM.                                           --
  ----------------------------------------------------------------------

  fsm_process : process(CLK)
  begin
    if rising_edge(CLK) then
      if (state = "00000") then
         if (reboot = '1') then
           state <= "00001";
         end if;
      elsif (state = DONE) then
         state <= state;
      else
        state <= state + "00001";
      end if;
    end if;
  end process fsm_process;

  ----------------------------------------------------------------------
  -- Control ICAP based on FSM state                                  --
  ----------------------------------------------------------------------

  --new_mode_reg <= '0' & internal_use & internal_mode & internal_vsel;
  -- New mode must be set to 0 in order for CWDT to operate properly and
  -- cover-off additional failure scenarios
  new_mode_reg <= '0' & '0' & internal_mode & internal_vsel;


  fsm_output_process : process(state, internal_addr, new_mode_reg)
  begin
    case state is
       when "00001" => icap_ce   <= '0'; 
                       icap_data <= X"AA"; -- sync H
       when "00010" => icap_ce   <= '0'; 
                       icap_data <= X"99"; -- sync L
       when "00011" => icap_ce   <= '0'; 
                       icap_data <= X"32"; -- t1w gen1 H
       when "00100" => icap_ce   <= '0'; 
                       icap_data <= X"61"; -- t1w gen1 L
       when "00101" => icap_ce   <= '0'; 
                       icap_data <= internal_addr(15 downto 8); -- user addr
       when "00110" => icap_ce   <= '0'; 
                       icap_data <= internal_addr(7 downto 0);  -- user addr
       when "00111" => icap_ce   <= '0'; 
                       icap_data <= X"32"; -- t1w gen2 H
       when "01000" => icap_ce   <= '0'; 
                       icap_data <= X"81"; -- t1w gen2 L
       when "01001" => icap_ce   <= '0'; 
                       icap_data <= internal_addr(31 downto 24); -- user addr
       when "01010" => icap_ce   <= '0'; 
                       icap_data <= internal_addr(23 downto 16); -- user addr
       when "01011" => icap_ce   <= '0'; 
                       icap_data <= X"32"; -- t1w mode H
       when "01100" => icap_ce   <= '0'; 
                       icap_data <= X"A1"; -- t1w mode L
       when "01101" => icap_ce   <= '0'; 
                       icap_data <= X"00"; -- reserved
       when "01110" => icap_ce   <= '0'; 
                       icap_data <= new_mode_reg; -- user mode
       when "01111" => icap_ce   <= '0'; 
                       icap_data <= X"30"; -- t1w cmd H
       when "10000" => icap_ce   <= '0'; 
                       icap_data <= X"A1"; -- t1w cmd L
       when "10001" => icap_ce   <= '0'; 
                       icap_data <= X"00"; -- reboot H
       when "10010" => icap_ce   <= '0'; 
                       icap_data <= X"0E"; -- reboot L
       when "10011" => icap_ce   <= '0'; 
                       icap_data <= X"20"; -- noop H
       when "10100" => icap_ce   <= '0';  
                       icap_data <= X"00"; -- noop L
       when others  => icap_ce   <= '1';
                       icap_data <= (others => '0');
    end case;
  end process fsm_output_process;

  ce <= icap_ce; -- active low
  data(0) <= icap_data(7); -- swizzled
  data(1) <= icap_data(6); -- swizzled
  data(2) <= icap_data(5); -- swizzled
  data(3) <= icap_data(4); -- swizzled
  data(4) <= icap_data(3); -- swizzled
  data(5) <= icap_data(2); -- swizzled
  data(6) <= icap_data(1); -- swizzled
  data(7) <= icap_data(0); -- swizzled
  write <= '0'; -- active low

end RTL;
