------------------------------------------------------------------------------
-- Copyright (c) 2008 Xilinx, Inc.
-- This design is confidential and proprietary of Xilinx, All Rights Reserved.
------------------------------------------------------------------------------
--   ____  ____
--  /   /\/   /
-- /___/  \  /   Vendor:        Xilinx
-- \   \   \/    Version:       1.0.0
--  \   \        Filename:      picojtag.vhd
--  /   /        Date Created:  April 22, 2008
-- /___/   /\    Last Modified: April 22, 2008
-- \   \  /  \
--  \___\/\___\
--
-- Devices:   Spartan-3 Generation FPGA
-- Purpose:   Failsafe multiboot  
-- Contact:   jimwe@xilinx.com
-- Reference: None
--
-- Revision History:
--   Rev 1.0.0 - (jimwe) First created April 22, 2008
--
------------------------------------------------------------------------------
--
-- LIMITED WARRANTY AND DISCLAIMER. These designs are provided to you "as is".
-- Xilinx and its licensors make and you receive no warranties or conditions,
-- express, implied, statutory or otherwise, and Xilinx specifically disclaims
-- any implied warranties of merchantability, non-infringement, or fitness for
-- a particular purpose. Xilinx does not warrant that the functions contained
-- in these designs will meet your requirements, or that the operation of
-- these designs will be uninterrupted or error free, or that defects in the
-- designs will be corrected. Furthermore, Xilinx does not warrant or make any
-- representations regarding use or the results of the use of the designs in
-- terms of correctness, accuracy, reliability, or otherwise.
--
-- LIMITATION OF LIABILITY. In no event will Xilinx or its licensors be liable
-- for any loss of data, lost profits, cost or procurement of substitute goods
-- or services, or for any special, incidental, consequential, or indirect
-- damages arising from the use or operation of the designs or accompanying
-- documentation, however caused and on any theory of liability. This
-- limitation will apply even if Xilinx has been advised of the possibility
-- of such damage. This limitation shall apply not-withstanding the failure
-- of the essential purpose of any limited remedies herein.
--
------------------------------------------------------------------------------
-- Copyright (c) 2008 Xilinx, Inc.
-- This design is confidential and proprietary of Xilinx, All Rights Reserved.
------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.all;

library UNISIM;
use UNISIM.VCOMPONENTS.all;

entity PICOJTAG is
  port (
    CLK           : in  std_logic;
    RESET_BUFFER  : in  std_logic;
    READ_BUFFER   : in  std_logic;
    WRITE_BUFFER  : in  std_logic;
    DATA_IN       : in  std_logic_vector(7 downto 0);
    DATA_OUT      : out std_logic_vector(7 downto 0);
    TBUFFER_DATA_PRESENT : out std_logic;
    TBUFFER_HALF_FULL    : out std_logic;
    TBUFFER_FULL         : out std_logic;
    RBUFFER_DATA_PRESENT : out std_logic;
    RBUFFER_HALF_FULL    : out std_logic;
    RBUFFER_FULL         : out std_logic 
  );
end PICOJTAG;

architecture RTL of PICOJTAG is

  component bbfifo_16x8 
    port (
      DATA_IN      : in  std_logic_vector(7 downto 0);
      DATA_OUT     : out std_logic_vector(7 downto 0);
      RESET        : in  std_logic;
      WRITE        : in  std_logic;
      READ         : in  std_logic;
      FULL         : out std_logic;
      HALF_FULL    : out std_logic;
      DATA_PRESENT : out std_logic;
      CLK          : in  std_logic);
  end component;
    
  signal update               : std_logic;
  signal shift                : std_logic;
  signal tdi                  : std_logic;
  signal tdo                  : std_logic;
  signal tck                  : std_logic;
  signal tck_g                : std_logic;
  signal juse                 : std_logic;
  signal capture              : std_logic;
  signal jshift_reg           : std_logic_vector(12 downto 0);
  signal jfifo_read           : std_logic;
  signal jfifo_write          : std_logic;
  signal jfifo_read_complete  : std_logic;
  signal jfifo_write_complete : std_logic;
  signal update_ff            : std_logic;
  signal tbuffer_dout         : std_logic_vector(7 downto 0);
  signal tbuffer_data_present_i : std_logic;
  signal tbuffer_half_full_i    : std_logic;
  signal tbuffer_full_i         : std_logic;
  signal rbuffer_data_present_i : std_logic;
  signal rbuffer_half_full_i    : std_logic;
  signal rbuffer_full_i         : std_logic;

begin

   -- Assign outputs
  TBUFFER_DATA_PRESENT <= tbuffer_data_present_i;
  TBUFFER_HALF_FULL    <= tbuffer_half_full_i;
  TBUFFER_FULL         <= tbuffer_full_i;
  RBUFFER_DATA_PRESENT <= rbuffer_data_present_i;
  RBUFFER_HALF_FULL    <= rbuffer_half_full_i;
  RBUFFER_FULL         <= rbuffer_full_i;

  -- Instantiate bscan primitive
  s3_bscan : BSCAN_SPARTAN3
    port map (
      TDO1    => tdo,
      TDO2    => '0',
      UPDATE  => update,
      SHIFT   => shift,
      RESET   => open,
      TDI     => tdi,
      SEL1    => juse,
      DRCK1   => tck,
      SEL2    => open,
      DRCK2   => open,
      CAPTURE => capture );
 
  -- Instantiate bufg for the boundary scan module
  jtag_clock : BUFG 
    port map (
      I => TCK,
      O => tck_g );

  -- The jtag shift register
    
  tdo <= jshift_reg(0);

  shift_process : process(tck_g)
  begin
    if rising_edge(tck_g) then
      if (capture = '1') then
        jshift_reg <= '0' & tbuffer_data_present_i & tbuffer_full_i &
                          rbuffer_data_present_i & rbuffer_full_i &
                          tbuffer_dout;
      else
        jshift_reg <= tdi & jshift_reg(12 downto 1);
      end if;
    end if;
  end process shift_process;

  -- Resynchronize from jtag clk domain to clk domain
  update_process : process(CLK)
  begin
    if rising_edge(CLK) then
      update_ff <= update;
    end if;
  end process update_process;

  -- Read strobe edge detection
  read_process : process(CLK)
  begin
    if rising_edge(CLK) then
      if (update_ff = '1') then
        jfifo_read <= not jfifo_read_complete and jshift_reg(8);
        jfifo_read_complete <= '1';
      else
        jfifo_read <= '0';
        jfifo_read_complete <= '0';
      end if;
    end if;
  end process read_process;

  -- Write strobe edge detection
  write_process : process(CLK)
  begin
    if rising_edge(CLK) then
      if (update_ff = '1') then
         jfifo_write <= not jfifo_write_complete and jshift_reg(9);
         jfifo_write_complete <= '1';
      else
         jfifo_write <= '0';
         jfifo_write_complete <= '0';
      end if;
    end if;
  end process write_process;

  -- Data fifos
  rBuffer : bbfifo_16x8 
    port map (
      DATA_IN      => jshift_reg(7 downto 0),
      DATA_OUT     => DATA_OUT,
      RESET        => RESET_BUFFER,
      WRITE        => jfifo_write,
      READ         => READ_BUFFER,
      FULL         => rbuffer_full_i,
      HALF_FULL    => rbuffer_half_full_i,
      DATA_PRESENT => rbuffer_data_present_i,
      CLK          => CLK );

  tBuffer : bbfifo_16x8 
    port map (
      DATA_IN      => DATA_IN,
      DATA_OUT     => TBUFFER_DOUT,
      RESET        => RESET_BUFFER,
      WRITE        => WRITE_BUFFER,
      READ         => jfifo_read,
      FULL         => tbuffer_full_i,
      HALF_FULL    => tbuffer_half_full_i,
      DATA_PRESENT => tbuffer_data_present_i,
      CLK          => CLK );

end RTL;
